"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Copyright (c) 2020 GitHub. This code is licensed under MIT license (see LICENSE(https://github.com/github/dynamic-template-engine/blob/master/LICENSE) for details)
const Handlebars = require("handlebars");
const FunctionalityError_1 = require("../../Error/FunctionalityError");
const TemplateError_1 = require("../../Error/TemplateError");
class HandleBarsTemplateEngine {
    constructor() {
        this.preCompiledTemplateMap = new Map();
    }
    /**
     * Registers the template with the template engine by compiling and storing the compiled method.
     * TODO :: Add parials template support
     *
     * @param {string} templateId - id used to store the precompiled template
     * @param {string} template - template to register
     * @param {JSON} options - provide options such as partial or not
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    registerTemplate(templateId, template, options) {
        // TODO :: Add parials template support using options
        let preCompiledTemplate;
        try {
            preCompiledTemplate = Handlebars.compile(template);
        }
        catch (error) {
            throw new TemplateError_1.TemplateParseError(error.message);
        }
        this.preCompiledTemplateMap.set(templateId, preCompiledTemplate);
    }
    /**
     * Apply the template using the data provided.
     *
     * @param {string} templateId - id with which the compiled template is stored
     * @param {JSON} dataModel - data to apply to the template
     */
    applyTemplate(templateId, dataModel) {
        const preCompiledTemplate = this.preCompiledTemplateMap.get(templateId);
        if (!preCompiledTemplate) {
            throw new TemplateError_1.TemplateNotFound(`No template found for the given templateId : ${templateId}`);
        }
        return preCompiledTemplate(dataModel);
    }
    /**
     * Register custom helper functions with template engine.
     *
     * @param {string} helperName name of the helper to register
     * @param {Handlebars.HelperDelegate} helperFunc the implementation of helper function
     */
    // eslint-disable-next-line class-methods-use-this
    registerHelper(helperName, helperFunc) {
        try {
            Handlebars.registerHelper(helperName, helperFunc);
        }
        catch (error) {
            throw new FunctionalityError_1.CustomHelperRegisterError(`Registration of custom helper: ${helperName} failed with ERROR: ${error.message} `);
        }
    }
    /**
     * Register custom tag with template engine.
     *
     * @throws FunctionalityNotSupportedError if the engine does not support custom tags/extensions
     */
    // eslint-disable-next-line class-methods-use-this
    registerTag() {
        throw new FunctionalityError_1.FunctionalityNotSupportedError('HandleBars does not support custom tags or extensions');
    }
}
exports.default = HandleBarsTemplateEngine;
//# sourceMappingURL=HandleBarsTemplateEngine.js.map