"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/** @copyright Copyright (c) 2020 GitHub. This code is licensed under MIT license (see LICENSE(https://github.com/github/dynamic-template-engine/blob/master/LICENSE) for details) */
const liquidjs_1 = require("liquidjs");
const FunctionalityError_1 = require("../../Error/FunctionalityError");
const TemplateError_1 = require("../../Error/TemplateError");
class LiquidTemplateEngine {
    constructor() {
        this.preCompiledTemplateMap = new Map();
        this.engine = new liquidjs_1.Liquid();
    }
    /**
     * Registers the template with the template engine by compiling and storing the compiled method.
     * TODO :: Add parials template support using options
     *
     * @param {string} templateId - id used to store the precompiled template
     * @param {string} template - template to register
     * @param {JSON} options - provide options such as partial or not
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    registerTemplate(templateId, template, options) {
        // TODO :: Add parials template support using options
        let preCompiledTemplate = [];
        try {
            preCompiledTemplate = this.engine.parse(template);
        }
        catch (error) {
            throw new TemplateError_1.TemplateParseError(error.message);
        }
        this.preCompiledTemplateMap.set(templateId, preCompiledTemplate);
    }
    /**
     * Apply the template using the data provided.
     *
     * @param {string} templateId - id with which the compiled template is stored
     * @param {JSON} dataModel - data to apply to the template
     */
    applyTemplate(templateId, dataModel) {
        const preCompiledTemplate = this.preCompiledTemplateMap.get(templateId);
        if (!preCompiledTemplate) {
            throw new TemplateError_1.TemplateNotFound(`No template found for the given templateId : ${templateId}`);
        }
        return this.engine.renderSync(preCompiledTemplate, dataModel);
    }
    /**
     * Register custom helper functions with template engine.
     *
     * @param {string} helperName name of the helper to register
     * @param {FilterImplOptions} helperFunc the implementation of helper function
     */
    // eslint-disable-next-line class-methods-use-this
    registerHelper(helperName, helperFunc) {
        try {
            this.engine.registerFilter(helperName, helperFunc);
        }
        catch (error) {
            throw new FunctionalityError_1.CustomHelperRegisterError(`Registration of custom helper: ${helperName} failed with ERROR: ${error.message} `);
        }
    }
    /**
     * Register custom tag with template engine.
     *
     * @param {string} tagName name of the tag to register
     * @param {TagImplOptions} tagOptions tagOptions specific to the template engine
     */
    // eslint-disable-next-line class-methods-use-this
    registerTag(tagName, tagOptions) {
        try {
            this.engine.registerTag(tagName, tagOptions);
        }
        catch (error) {
            throw new FunctionalityError_1.CustomTagRegisterError(`Registration of custom Tag: ${tagName} failed with ERROR: ${error.message}`);
        }
    }
}
exports.default = LiquidTemplateEngine;
//# sourceMappingURL=LiquidTemplateEngine.js.map