"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Copyright (c) 2020 GitHub. This code is licensed under MIT license (see LICENSE(https://github.com/github/dynamic-template-engine/blob/master/LICENSE) for details)
const path = require("path");
const TemplateEngineFactory_1 = require("./Template/Core/TemplateEngineFactory");
const CardRenderer_1 = require("./Transformer/CardRenderer/CardRenderer");
const EventTransformer_1 = require("./Transformer/EventTransformer/EventTransformer");
const TemplateError_1 = require("./Error/TemplateError");
const FileError_1 = require("./Error/FileError");
const Utility_1 = require("./Utility/Utility");
const FunctionalityError_1 = require("./Error/FunctionalityError");
/**
 * Template Manager provides methods to setup the template configuration
 * intializes template engines, registers all the templates provided by the config
 */
class TemplateManager {
    /**
     * Sets up the templates, picking from the path provided,
     * registering them to appropriate engines
     *
     * @param {string} configFilePath - path of the config file
     * @returns {boolean} true if setup successful
     * @throws Error if setup fails
     */
    static async setupTemplateConfiguration(configFilePath, customOptions) {
        var _a;
        try {
            const baseUrl = configFilePath.substring(0, configFilePath.lastIndexOf(path.sep));
            const transformerConfig = await this.readConfigFile(configFilePath, false);
            (_a = customOptions === null || customOptions === void 0 ? void 0 : customOptions.engineOptions) === null || _a === void 0 ? void 0 : _a.forEach(engineOption => {
                this.registerHelpersAndTags(engineOption);
            });
            await this.registerAllTemplates(baseUrl, new CardRenderer_1.default(), transformerConfig.cardRenderer);
            await this.registerAllTemplates(baseUrl, new EventTransformer_1.default(), transformerConfig.eventTransformer);
        }
        catch (error) {
            const defaultMessage = `Setup failed for the config file path: ${configFilePath} \n ERROR: ${error.message}`;
            throw TemplateManager.handleErrors(error, defaultMessage);
        }
        return true;
    }
    /**
     * Sets up the templates, picking from the path provided,
     * registering them to appropriate engines
     *
     * @param {string} repo - repo name ex. user/repo
     * @param {string} branch - ex. master
     * @param {string} configName config file name in the repo root folder
     * @returns {boolean} true if setup succesful
     * @throws Error if setup fails
     */
    static async setupTemplateConfigurationFromRepo(repo, branch, configName, customOptions) {
        var _a;
        const baseUrl = `https://raw.githubusercontent.com/${repo}/${branch}`;
        try {
            const transformerConfig = await this.readConfigFile(`${baseUrl}/${configName}.json`, true);
            (_a = customOptions === null || customOptions === void 0 ? void 0 : customOptions.engineOptions) === null || _a === void 0 ? void 0 : _a.forEach(engineOption => {
                this.registerHelpersAndTags(engineOption);
            });
            await this.registerAllTemplates(baseUrl, new CardRenderer_1.default(), transformerConfig.cardRenderer);
            await this.registerAllTemplates(baseUrl, new EventTransformer_1.default(), transformerConfig.eventTransformer);
        }
        catch (error) {
            const defaultMessage = `Setup failed for repo: ${repo} and branch: ${branch}. \n ERROR: ${error.message}`;
            throw TemplateManager.handleErrors(error, defaultMessage);
        }
        return true;
    }
    /**
     * Read config file and deserialize the file appropriately
     *
     * @param {string} filePath - file path of the config
     * @param {boolean} fromRepo - specifies if file from repo or from local machine
     */
    static async readConfigFile(filePath, fromRepo) {
        const data = await Utility_1.default.fetchFile(fromRepo, filePath);
        try {
            return JSON.parse(data.toString());
        }
        catch (error) {
            throw new FileError_1.FileParseError(`Unable to parse config file from path 
      ${filePath}, original error message: ${error.message}`);
        }
    }
    /**
     * Register all templates provided in the transformerConfig
     *
     * @param {string} baseUrl - base url for the template files
     * @param {string} transformer - transformer whith which template should be registered
     * @param {BaseTransformConfigEntry} transformerConfigs - the template transformer configs
     */
    static async registerAllTemplates(baseUrl, transformer, transformerConfigs) {
        // eslint-disable-next-line no-restricted-syntax
        for (const element of transformerConfigs) {
            try {
                // eslint-disable-next-line no-await-in-loop
                await transformer.registerTemplate(baseUrl, element);
            }
            catch (error) {
                if (error instanceof TemplateError_1.TemplateParseError) {
                    throw new TemplateError_1.TemplateParseError(`Failed to parse template with name: ${element.TemplateName} 
          for source type: ${element.SourceType} and template type: ${element.TemplateType} with original message: ${error.message}`);
                }
                else {
                    throw error;
                }
            }
        }
    }
    /**
     * Registers custom helpers and tags for a specific template engine
     *
     * @param {CustomEngineOptions} engineOption template type and the list of
     * custom helpers and tag to register
     */
    static registerHelpersAndTags(engineOption) {
        const engine = TemplateEngineFactory_1.default.getInstance().getTemplateEngine(engineOption.templateType);
        const helpers = engineOption.customHelpers;
        if (helpers) {
            Object.keys(helpers).forEach(helperName => {
                engine.registerHelper(helperName, helpers[helperName]);
            });
        }
        const tags = engineOption.customTags;
        if (tags) {
            Object.keys(tags).forEach(tagName => {
                engine.registerTag(tagName, tags[tagName]);
            });
        }
    }
    /**
     * Determine the correct error type and respond accordingly.
     *
     * @param {Error} error error to be analyzed
     * @param {string} defaultMessage message to show when error is not a known type
     * @returns {Error} correct error to throw
     */
    static handleErrors(error, defaultMessage) {
        if (error instanceof TemplateError_1.TemplateEngineNotFound || error instanceof TemplateError_1.TemplateParseError
            || error instanceof FileError_1.FileParseError || error instanceof FileError_1.EmptyFileError
            || error instanceof FunctionalityError_1.FunctionalityNotSupportedError
            || error instanceof FunctionalityError_1.CustomHelperRegisterError
            || error instanceof FunctionalityError_1.CustomTagRegisterError) {
            // eslint-disable-next-line @typescript-eslint/no-throw-literal
            return error;
        }
        else {
            return new Error(defaultMessage);
        }
    }
}
exports.default = TemplateManager;
//# sourceMappingURL=TemplateManager.js.map