"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Copyright (c) 2020 GitHub. This code is licensed under MIT license (see LICENSE(https://github.com/github/dynamic-template-engine/blob/master/LICENSE) for details)
const Transformer_1 = require("../Core/Transformer");
const Utility_1 = require("../../Utility/Utility");
const TemplateError_1 = require("../../Error/TemplateError");
/**
 * EventTransformer provides ConstructEventJson method to render
 * a template with provided event data.
 */
class EventTransformer extends Transformer_1.default {
    /**
     * Construct a new Event Json using template and provided event data.
     *
     * @param {TemplateType} templateType - template engine to use ex. HandleBars, Liquid
     * @param {string} sourceType - ex. PullRequest_Opened, Issue_opened
     * @param {JSON} eventJson - event data to plug in the template
     * @returns {string} rendered template as a string
     * @throws Error if unable to apply template
     */
    ConstructEventJson(templateType, sourceType, eventJson) {
        const key = Utility_1.default.keyGenerator(EventTransformer.KEY_PREFIX, templateType, sourceType);
        try {
            return this.applyTemplate(templateType, key, eventJson);
        }
        catch (error) {
            if (error instanceof TemplateError_1.TemplateNotFound) {
                throw new TemplateError_1.TemplateNotFound(`No template found for Template Type: ${templateType} and Source Type: ${sourceType}`);
            }
            else if (error instanceof TemplateError_1.TemplateEngineNotFound) {
                throw error;
            }
            else {
                throw new TemplateError_1.TemplateRenderError(`Error applying template for Template Type: ${templateType} 
        and Source Type: ${sourceType} with error message ${error.message}`);
            }
        }
    }
    /**
     * Register a template with the correct engine based on the template config provided.
     * *** Internal function not exposed outside the package ***
     *
     * @internal
     * @param {string} baseUrl - location of the template file
     * @param {EventTransformConfigEntry} transformConfig - config details of the template to register
     */
    async registerTemplate(baseUrl, transformConfig) {
        const isHttpCall = baseUrl.startsWith('https://');
        const basepath = `${baseUrl}/EventTemplate`;
        const path = `${basepath}/${transformConfig.TemplateType}/${transformConfig.TemplateName}`;
        const key = Utility_1.default.keyGenerator(EventTransformer.KEY_PREFIX, transformConfig.TemplateType, transformConfig.SourceType);
        await this.readAndRegisterTemplate(isHttpCall, path, key, transformConfig.TemplateType);
    }
}
exports.default = EventTransformer;
EventTransformer.KEY_PREFIX = 'event';
//# sourceMappingURL=EventTransformer.js.map