"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Copyright (c) 2020 GitHub. This code is licensed under MIT license (see LICENSE(https://github.com/github/dynamic-template-engine/blob/master/LICENSE) for details)
const https = require("https");
const fs = require("fs");
const FileError_1 = require("../Error/FileError");
/**
 * Utility functions available to the whole code base.
 */
class Utility {
    /**
     * A synchoronous http call.
     *
     * @param {string} url - url to make the http call to
     * @returns {string} response string
     */
    static httpSync(url) {
        return new Promise((resolve, reject) => {
            https.get(url, (response) => {
                const { statusCode } = response;
                if (statusCode !== 200) {
                    reject(new Error(`Http Call failed. Status Code: ${statusCode}`));
                }
                // eslint-disable-next-line @typescript-eslint/naming-convention
                const chunks_of_data = [];
                response.on('data', (fragments) => {
                    chunks_of_data.push(fragments);
                });
                response.on('end', () => {
                    // eslint-disable-next-line @typescript-eslint/naming-convention
                    const response_body = Buffer.concat(chunks_of_data);
                    resolve(response_body.toString());
                });
                response.on('error', (error) => {
                    reject(error);
                });
            });
        });
    }
    /**
     * Fetch file either from local machine or using an http call.
     *
     * @param {boolean} isHttpCall - is an http call or a local machine lookup
     * @param {string} filePath - the path of the file to read
     */
    static async fetchFile(isHttpCall, filePath) {
        let file = '';
        try {
            if (isHttpCall) {
                file = await this.httpSync(filePath);
            }
            else {
                file = fs.readFileSync(`${filePath}`).toString();
            }
        }
        catch (error) {
            throw new FileError_1.FileReadError(`Could not read file with 
        file path: ${filePath} \n ERROR: ${error.message}`);
        }
        if (!file || file.length <= 0) {
            throw new FileError_1.EmptyFileError(`Empty file at path: ${filePath}`);
        }
        return file;
    }
    /**
     * Creates a key based on all strings passed.
     * Implementation can be changed to create a hash of all strings rather than just concating.
     *
     * @param {string[]} allStrings - all strings to create a unique key
     * @returns {string} generated key
     */
    static keyGenerator(...allStrings) {
        let genratedKey = '';
        allStrings.forEach(element => {
            genratedKey += `${element}.`;
        });
        return genratedKey.slice(0, genratedKey.length - 1);
    }
}
exports.default = Utility;
//# sourceMappingURL=Utility.js.map